cma.fdr <- function(gene.sizes,
                    passenger.rates = t(data.frame(.55*rep(1.0e-6,25))),
                    cma.data,
                    coverage,
                    scores = c("CaMP","logLRT"),
                    allgenes=FALSE,
                    estimate.p0=FALSE,
                    p0.step=1,
                    p0=1,
                    eliminate.noval=TRUE,
                    filter.threshold=0, 
                    filter.above=0,
                    filter.below=0,
                    filter.mutations=0, 
                    aa=1e-10, 
                    bb=1e-10,
                    priorH0=1-500/13020, 
                    prior.a0=100,
                    prior.a1=5,
                    prior.fold=10,
                    M=100){

  ##scores <- c("logLRT", "neglogPg")
  ##scores <- c("logLRT")
  if (estimate.p0) { allgenes==TRUE; eliminate.noval=FALSE }
  if (allgenes) {
    cma.data <- add.zeroes.cma.data(cma.data=cma.data,
                                    gene.sizes=gene.sizes,
                                    coverage=coverage)
    filter.above=0
    filter.below=0
    eliminate.noval==FALSE
   }
    
  number.genes=nrow(gene.sizes)
  
  # observed statistics
  obs.score <-
    cma.scores(cma.data,
               passenger.rates,
               number.genes,
               compute.poisson.BF=FALSE,
               compute.binomial.posterior=FALSE,
               filter.above=filter.above,
               filter.below=filter.below,
               filter.threshold=filter.threshold,
               aa=aa, 
               bb=bb,
               priorH0=priorH0, 
               prior.a0=prior.a0,
               prior.a1=prior.a1,
               ##prior.fold=prior.fold)[,c("CaMP",scores)]
               prior.fold=prior.fold)[,scores,drop=FALSE]
  
  ColMutDisc <- grep("MutationsDiscovery",colnames(cma.data))
  ColMutVali <- grep("MutationsValidation",colnames(cma.data))

  pass <- rep(TRUE,nrow(obs.score))
  if ( eliminate.noval ) {
    DiscoveryMutations <- apply(cma.data[,ColMutDisc],1,sum)
    ValidationMutations <- apply(cma.data[,ColMutVali],1,sum)
    pass <- DiscoveryMutations > 0 & ValidationMutations > 0
  }
  if (filter.mutations>0) {
    pass <- pass & ( apply(cma.data[,c(ColMutDisc,ColMutVali)],1,sum) >= filter.mutations )
  }

  obs.score <- obs.score[pass,,drop=FALSE]

  # null distribution
  null.score <- data.frame()
  for (m in 1:M){
    null <- cma.simulator(gene.sizes,
                          passenger.rates,
                          coverage=coverage,
                          eliminate.noval=eliminate.noval,
                          filter.mutations=filter.mutations)

    ms <- cma.scores(cma.data=null,
                     passenger.rates,
                     number.genes,
                     compute.poisson.BF=FALSE,
                     compute.binomial.posterior=FALSE,
                     filter.above=filter.above,
                     filter.below=filter.below,
                     filter.threshold=filter.threshold,
                     aa=aa, 
                     bb=bb,
                     priorH0=priorH0, 
                     prior.a0=prior.a0,
                     prior.a1=prior.a1,
                     prior.fold=prior.fold
                     )[,scores,drop=FALSE]
    
    null.score <- rbind(null.score,ms) 
  }
  
  if (nrow(null.score)==0) stop("no null genes made it through to be validated; increase M")
  out <- list()

  ##browser()
  
  for (ss in 1:length(scores) ){
    ##obs <- obs.score[,ss+1]; names(obs) <- rownames(obs.score)

    obs <- obs.score[,ss]
    names(obs) <- rownames(obs.score)
    
    obs[obs == Inf] = 10^6
    
    tmp <- ebfdr(obs.score=obs,
                 ##null.score=null.score[,ss],
                 null.score=null.score,
                 mass=M,
                 estimate.p0 = estimate.p0,
                 p0.step = p0.step,
                 p0=p0)
    ##tmp <- cbind( obs.score[rownames(tmp),"CaMP"], tmp )
    ##colnames(tmp)[1] <- "CaMP"
    out[[ss]] <- tmp
  }
  names(out) <- scores
  
  return(out)
}
